@@ -755,6 +755,7 @@ class AssemblylineDatastore(object):
             "tags": set(),
             "sections": set()
         }
+        signatures = {}
 
         if len(keys) == 0:
             return out
@@ -800,24 +801,29 @@ class AssemblylineDatastore(object):
                     heur_name = section['heuristic']['name']
 
                     # Get the heuristics data
-                    if section['heuristic']['score'] < 0:
-                        h_type = "safe"
-                    elif section['heuristic']['score'] < 300:
-                        h_type = "info"
-                    elif section['heuristic']['score'] < 1000:
-                        h_type = "suspicious"
-                    else:
+                    if section['heuristic']['score'] >= config.submission.verdicts.malicious:
                         h_type = "malicious"
+                    elif section['heuristic']['score'] >= config.submission.verdicts.suspicious:
+                        h_type = "suspicious"
+                    elif section['heuristic']['score'] >= config.submission.verdicts.info:
+                        h_type = "info"
+                    else:
+                        h_type = "safe"
 
                     cache_key = f"{heur_id}_{key}"
+                    signatures.setdefault(cache_key, [])
+
                     if cache_key not in done_map['heuristics']:
                         out['heuristics'][h_type].append({
                             'heur_id': heur_id,
                             'name': heur_name,
-                            'key': key
+                            'key': key,
+                            'signatures': []
                         })
                         done_map['heuristics'].add(cache_key)
 
+                    signatures[cache_key].extend([s['name'] for s in section['heuristic']['signature']])
+
                     if keep_heuristic_sections:
                         # Set defaults
                         out['heuristic_sections'].setdefault(heur_id, [])
@@ -882,6 +888,11 @@ class AssemblylineDatastore(object):
                                 })
                                 done_map['tags'].add(cache_key)
 
+            for htype in out['heuristics']:
+                for heur in out['heuristics'][htype]:
+                    cache_key = f"{heur_id}_{key}"
+                    heur['signatures'].extend(signatures.get(cache_key, []))
+
         return out
 
     @elasticapm.capture_span(span_type='datastore')
