@@ -3,7 +3,7 @@ from typing import Dict, List
 from assemblyline import odm
 from assemblyline.odm.models.service import EnvironmentVariable
 
-OAUTH_AUTO_PROPERTY_TYPE = ['access', 'classification', 'role']
+AUTO_PROPERTY_TYPE = ['access', 'classification', 'type', 'role', 'remove_role']
 
 
 @odm.model(index=False, store=False, description="Password Requirement")
@@ -83,6 +83,14 @@ DEFAULT_SIGNUP = {
 }
 
 
+@odm.model(index=False, store=False)
+class AutoProperty(odm.Model):
+    field: str = odm.Keyword(description="Field to apply `pattern` to")
+    pattern: str = odm.Keyword(description="Regex pattern for auto-prop assignment")
+    type: str = odm.Enum(AUTO_PROPERTY_TYPE, description="Type of property assignment on pattern match")
+    value: str = odm.Keyword(description="Assigned property value")
+
+
 @odm.model(index=False, store=False, description="LDAP Configuration")
 class LDAP(odm.Model):
     enabled: bool = odm.Boolean(description="Should LDAP be enabled or not?")
@@ -91,6 +99,8 @@ class LDAP(odm.Model):
     bind_pass: str = odm.Optional(odm.Keyword(), description="Password used to query the LDAP server")
     auto_create: bool = odm.Boolean(description="Auto-create users if they are missing")
     auto_sync: bool = odm.Boolean(description="Should we automatically sync with LDAP server on each login?")
+    auto_properties: List[AutoProperty] = odm.List(odm.Compound(AutoProperty), default=[],
+                                                   description="Automatic role and classification assignments")
     base: str = odm.Keyword(description="Base DN for the users")
     classification_mappings: Dict[str, str] = odm.Any(description="Classification mapping")
     email_field: str = odm.Keyword(description="Name of the field containing the email address")
@@ -110,23 +120,25 @@ class LDAP(odm.Model):
 
 DEFAULT_LDAP = {
     "enabled": False,
-    "admin_dn": None,
     "bind_user": None,
     "bind_pass": None,
     "auto_create": True,
     "auto_sync": True,
+    "auto_properties": [],
     "base": "ou=people,dc=assemblyline,dc=local",
-    "classification_mappings": {},
     "email_field": "mail",
     "group_lookup_query": "(&(objectClass=Group)(member=%s))",
     "image_field": "jpegPhoto",
     "image_format": "jpeg",
     "name_field": "cn",
-    "signature_importer_dn": None,
-    "signature_manager_dn": None,
     "uid_field": "uid",
     "uri": "ldap://localhost:389",
 
+    # Deprecated
+    "admin_dn": None,
+    "classification_mappings": {},
+    "signature_importer_dn": None,
+    "signature_manager_dn": None,
 }
 
 
@@ -150,14 +162,6 @@ DEFAULT_INTERNAL = {
 }
 
 
-@odm.model(index=False, store=False)
-class OAuthAutoProperty(odm.Model):
-    field: str = odm.Keyword(description="Field to apply `pattern` to")
-    pattern: str = odm.Keyword(description="Regex pattern for auto-prop assignment")
-    type: str = odm.Enum(OAUTH_AUTO_PROPERTY_TYPE, description="Type of property assignment on pattern match")
-    value: str = odm.Keyword(description="Assigned property value")
-
-
 @odm.model(index=False, store=False, description="App provider")
 class AppProvider(odm.Model):
     access_token_url: str = odm.Keyword(description="URL used to get the access token")
@@ -173,8 +177,8 @@ class AppProvider(odm.Model):
 class OAuthProvider(odm.Model):
     auto_create: str = odm.Boolean(default=True, description="Auto-create users if they are missing")
     auto_sync: str = odm.Boolean(default=False, description="Should we automatically sync with OAuth provider?")
-    auto_properties: List[OAuthAutoProperty] = odm.List(odm.Compound(OAuthAutoProperty), default=[],
-                                                        description="Automatic role and classification assignments")
+    auto_properties: List[AutoProperty] = odm.List(odm.Compound(AutoProperty), default=[],
+                                                   description="Automatic role and classification assignments")
     app_provider: AppProvider = odm.Optional(odm.Compound(AppProvider))
     uid_randomize: str = odm.Boolean(default=False,
                                      description="Should we generate a random username for the authenticated user?")
@@ -211,6 +215,10 @@ class OAuthProvider(odm.Model):
         odm.Keyword(),
         description="Name of the field in the list of groups that contains the name of the group")
     use_new_callback_format: str = odm.Boolean(default=False, description="Should we use the new callback method?")
+    allow_external_tokens: bool = odm.Boolean(
+        default=False, description="Should token provided to the login API directly be use for authentication?")
+    external_token_alternate_audiences: bool = odm.List(
+        odm.Keyword(), default=[], description="List of valid alternate audiences for the external token.")
 
 
 DEFAULT_OAUTH_PROVIDER_AZURE = {
@@ -357,6 +365,8 @@ class Expiry(odm.Model):
     workers = odm.Integer(description="Number of concurrent workers")
     delete_workers = odm.Integer(description="Worker processes for file storage deletes.")
     iteration_max_tasks = odm.Integer(description="How many query chunks get run per iteration.")
+    delete_batch_size = odm.Integer(description="How large a batch get deleted per iteration.")
+    archive_batch_size = odm.Integer(description="How large a batch get archived per iteration.")
 
 
 DEFAULT_EXPIRY = {
@@ -367,6 +377,8 @@ DEFAULT_EXPIRY = {
     'workers': 20,
     'delete_workers': 2,
     'iteration_max_tasks': 20,
+    'delete_batch_size': 2000,
+    'archive_batch_size': 5000,
 }
 
 
@@ -503,6 +515,19 @@ DEFAULT_REDIS = {
 }
 
 
+@odm.model(index=False, store=False, description="A configuration for mounting existing volumes to a container")
+class Mount(odm.Model):
+    name: str = odm.Keyword(description="Name of volume mount")
+    path: str = odm.Text(description="Target mount path")
+    read_only: bool = odm.Boolean(default=True, description="Should this be mounted as read-only?")
+    privileged_only: bool = odm.Boolean(default=False,
+                                        description="Should this mount only be available for privileged services?")
+
+    # ConfigMap-specific
+    config_map: str = odm.Optional(odm.Keyword(description="Name of ConfigMap (Kubernetes only)"))
+    key: str = odm.Optional(odm.Keyword(), description="Key of ConfigMap (Kubernetes only)")
+
+
 @odm.model(index=False, store=False,
            description="A set of default values to be used running a service when no other value is set")
 class ScalerServiceDefaults(odm.Model):
@@ -512,6 +537,8 @@ class ScalerServiceDefaults(odm.Model):
     min_instances: int = odm.Integer(description="The minimum number of service instances to be running")
     environment: List[EnvironmentVariable] = odm.List(odm.Compound(EnvironmentVariable), default=[],
                                                       description="Environment variables to pass onto services")
+    mounts: List[Mount] = odm.List(odm.Compound(Mount), default=[],
+                                   description="A list of volume mounts for every service")
 
 
 @odm.model(index=False, store=False)
@@ -520,6 +547,8 @@ class Scaler(odm.Model):
                                                            description="Defaults Scaler will assign to a service.")
     cpu_overallocation: float = odm.Float(description="Percentage of CPU overallocation")
     memory_overallocation: float = odm.Float(description="Percentage of RAM overallocation")
+    overallocation_node_limit = odm.Optional(odm.Integer(description="If the system has this many nodes or "
+                                                                     "more overallocation is ignored"))
     additional_labels: List[str] = odm.Optional(
         odm.List(odm.Text()), description="Additional labels to be applied to services('=' delimited)")
 
@@ -528,6 +557,7 @@ DEFAULT_SCALER = {
     'additional_labels': None,
     'cpu_overallocation': 1,
     'memory_overallocation': 1,
+    'overallocation_node_limit': None,
     'service_defaults': {
         'growth': 60,
         'shrink': 30,
@@ -664,7 +694,7 @@ class Datastore(odm.Model):
 
 
 DEFAULT_DATASTORE = {
-    "hosts": ["http://elastic:devpass@localhost"],
+    "hosts": ["http://elastic:devpass@localhost:9200"],
     "ilm": DEFAULT_ILM,
     "type": "elasticsearch",
 }
@@ -737,6 +767,7 @@ SERVICE_CATEGORIES = [
     'External',
     'Extraction',
     'Filtering',
+    'Internet Connected',
     'Networking',
     'Static Analysis',
 ]
@@ -1051,6 +1082,23 @@ EXAMPLE_SHA256_SOURCE_MB = {
 }
 
 
+@odm.model(index=False, store=False,
+           description="Minimum score value to get the specified verdict, otherwise the file is considered safe.")
+class Verdicts(odm.Model):
+    info: int = odm.Integer(description="Minimum score for the verdict to be Informational.")
+    suspicious: int = odm.Integer(description="Minimum score for the verdict to be Suspicious.")
+    highly_suspicious: int = odm.Integer(description="Minimum score for the verdict to be Highly Suspicious.")
+    malicious: int = odm.Integer(description="Minimum score for the verdict to be Malicious.")
+
+
+DEFAULT_VERDICTS = {
+    'info': 0,
+    'suspicious': 300,
+    'highly_suspicious': 700,
+    'malicious': 1000
+}
+
+
 @odm.model(index=False, store=False,
            description="Default values for parameters for submissions that may be overridden on a per submission basis")
 class Submission(odm.Model):
@@ -1068,6 +1116,8 @@ class Submission(odm.Model):
         default=[], description="List of external source to fetch file via their SHA256 hashes")
     tag_types = odm.Compound(TagTypes, default=DEFAULT_TAG_TYPES,
                              description="Tag types that show up in the submission summary")
+    verdicts = odm.Compound(Verdicts, default=DEFAULT_VERDICTS,
+                            description="Minimum score value to get the specified verdict.")
 
 
 DEFAULT_SUBMISSION = {
@@ -1080,7 +1130,8 @@ DEFAULT_SUBMISSION = {
     'max_metadata_length': 4096,
     'max_temp_data_length': 4096,
     'sha256_sources': [],
-    'tag_types': DEFAULT_TAG_TYPES
+    'tag_types': DEFAULT_TAG_TYPES,
+    'verdicts': DEFAULT_VERDICTS
 }
 
 
